@@ -21,6 +21,7 @@
 #include "zend_interfaces.h"
 #include "zend_exceptions.h"
 #include "zend_interfaces_arginfo.h"
+#include "zend_property_hooks.h"
 
 ZEND_API zend_class_entry *zend_ce_traversable;
 ZEND_API zend_class_entry *zend_ce_aggregate;
@@ -198,8 +199,7 @@ ZEND_API HashTable *zend_user_it_get_gc(zend_object_iterator *_iter, zval **tabl
 
 static const zend_object_iterator_funcs zend_interface_iterator_funcs_iterator = {
 	zend_user_it_dtor,
-	// FIXME: Adjust the actual function prototype in zend_object_iterator_funcs
-	(int (*)(zend_object_iterator *)) zend_user_it_valid,
+	zend_user_it_valid,
 	zend_user_it_get_current_data,
 	zend_user_it_get_current_key,
 	zend_user_it_move_forward,
@@ -304,7 +304,9 @@ static int zend_implement_aggregate(zend_class_entry *interface, zend_class_entr
 	funcs_ptr->zf_new_iterator = zend_hash_str_find_ptr(
 		&class_type->function_table, "getiterator", sizeof("getiterator") - 1);
 
-	if (class_type->get_iterator && class_type->get_iterator != zend_user_it_get_new_iterator) {
+	if (class_type->get_iterator
+	 && class_type->get_iterator != zend_user_it_get_new_iterator
+	 && class_type->get_iterator != zend_hooked_object_get_iterator) {
 		/* get_iterator was explicitly assigned for an internal class. */
 		if (!class_type->parent || class_type->parent->get_iterator != class_type->get_iterator) {
 			ZEND_ASSERT(class_type->type == ZEND_INTERNAL_CLASS);
@@ -351,7 +353,9 @@ static int zend_implement_iterator(zend_class_entry *interface, zend_class_entry
 	funcs_ptr->zf_next = zend_hash_str_find_ptr(
 		&class_type->function_table, "next", sizeof("next") - 1);
 
-	if (class_type->get_iterator && class_type->get_iterator != zend_user_it_get_iterator) {
+	if (class_type->get_iterator
+	 && class_type->get_iterator != zend_user_it_get_iterator
+	 && class_type->get_iterator != zend_hooked_object_get_iterator) {
 		if (!class_type->parent || class_type->parent->get_iterator != class_type->get_iterator) {
 			/* get_iterator was explicitly assigned for an internal class. */
 			ZEND_ASSERT(class_type->type == ZEND_INTERNAL_CLASS);
