@@ -28,7 +28,6 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_class_CURLStringFile___construct, 0, 0, 2)
 	ZEND_ARG_TYPE_INFO_WITH_DEFAULT_VALUE(0, mime, IS_STRING, 0, "\"application/octet-stream\"")
 ZEND_END_ARG_INFO()
 
-
 ZEND_METHOD(CURLFile, __construct);
 ZEND_METHOD(CURLFile, getFilename);
 ZEND_METHOD(CURLFile, getMimeType);
@@ -37,7 +36,6 @@ ZEND_METHOD(CURLFile, setMimeType);
 ZEND_METHOD(CURLFile, setPostFilename);
 ZEND_METHOD(CURLStringFile, __construct);
 
-
 static const zend_function_entry class_CURLFile_methods[] = {
 	ZEND_ME(CURLFile, __construct, arginfo_class_CURLFile___construct, ZEND_ACC_PUBLIC)
 	ZEND_ME(CURLFile, getFilename, arginfo_class_CURLFile_getFilename, ZEND_ACC_PUBLIC)
@@ -48,7 +46,6 @@ static const zend_function_entry class_CURLFile_methods[] = {
 	ZEND_FE_END
 };
 
-
 static const zend_function_entry class_CURLStringFile_methods[] = {
 	ZEND_ME(CURLStringFile, __construct, arginfo_class_CURLStringFile___construct, ZEND_ACC_PUBLIC)
 	ZEND_FE_END
@@ -59,8 +56,7 @@ static zend_class_entry *register_class_CURLFile(void)
 	zend_class_entry ce, *class_entry;
 
 	INIT_CLASS_ENTRY(ce, "CURLFile", class_CURLFile_methods);
-	class_entry = zend_register_internal_class_ex(&ce, NULL);
-	class_entry->ce_flags |= ZEND_ACC_NOT_SERIALIZABLE;
+	class_entry = zend_register_internal_class_with_flags(&ce, NULL, ZEND_ACC_NOT_SERIALIZABLE);
 
 	zval property_name_default_value;
 	ZVAL_EMPTY_STRING(&property_name_default_value);
@@ -88,7 +84,7 @@ static zend_class_entry *register_class_CURLStringFile(void)
 	zend_class_entry ce, *class_entry;
 
 	INIT_CLASS_ENTRY(ce, "CURLStringFile", class_CURLStringFile_methods);
-	class_entry = zend_register_internal_class_ex(&ce, NULL);
+	class_entry = zend_register_internal_class_with_flags(&ce, NULL, 0);
 
 	zval property_data_default_value;
 	ZVAL_UNDEF(&property_data_default_value);
