@@ -16,7 +16,7 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include "php.h"
@@ -30,11 +30,9 @@
 * Since:
 */
 
-extern zend_class_entry *dom_domexception_class_entry;
-
-void php_dom_throw_error_with_message(int error_code, char *error_message, int strict_error) /* {{{ */
+void php_dom_throw_error_with_message(dom_exception_code error_code, const char *error_message, bool strict_error) /* {{{ */
 {
-	if (strict_error == 1) {
+	if (strict_error) {
 		zend_throw_exception(dom_domexception_class_entry, error_message, error_code);
 	} else {
 		php_libxml_issue_error(E_WARNING, error_message);
@@ -43,9 +41,9 @@ void php_dom_throw_error_with_message(int error_code, char *error_message, int s
 /* }}} */
 
 /* {{{ php_dom_throw_error */
-void php_dom_throw_error(int error_code, int strict_error)
+void php_dom_throw_error(dom_exception_code error_code, bool strict_error)
 {
-	char *error_message;
+	const char *error_message;
 
 	switch (error_code)
 	{
