@@ -16,13 +16,13 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include "php.h"
 #if defined(HAVE_LIBXML) && defined(HAVE_DOM)
 #include "php_dom.h"
-#include "dom_ce.h"
+#include "dom_properties.h"
 
 /*
 * class DOMText extends DOMCharacterData
@@ -43,10 +43,10 @@ PHP_METHOD(DOMText, __construct)
 		RETURN_THROWS();
 	}
 
-	nodep = xmlNewText((xmlChar *) value);
+	nodep = xmlNewText(BAD_CAST value);
 
 	if (!nodep) {
-		php_dom_throw_error(INVALID_STATE_ERR, 1);
+		php_dom_throw_error(INVALID_STATE_ERR, true);
 		RETURN_THROWS();
 	}
 
@@ -64,17 +64,11 @@ readonly=yes
 URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-Text3-wholeText
 Since: DOM Level 3
 */
-int dom_text_whole_text_read(dom_object *obj, zval *retval)
+zend_result dom_text_whole_text_read(dom_object *obj, zval *retval)
 {
-	xmlNodePtr node;
-	xmlChar *wholetext = NULL;
+	DOM_PROP_NODE(xmlNodePtr, node, obj);
 
-	node = dom_object_get_node(obj);
-
-	if (node == NULL) {
-		php_dom_throw_error(INVALID_STATE_ERR, 1);
-		return FAILURE;
-	}
+	smart_str str = {0};
 
 	/* Find starting text node */
 	while (node->prev && ((node->prev->type == XML_TEXT_NODE) || (node->prev->type == XML_CDATA_SECTION_NODE))) {
@@ -83,16 +77,13 @@ int dom_text_whole_text_read(dom_object *obj, zval *retval)
 
 	/* concatenate all adjacent text and cdata nodes */
 	while (node && ((node->type == XML_TEXT_NODE) || (node->type == XML_CDATA_SECTION_NODE))) {
-		wholetext = xmlStrcat(wholetext, node->content);
+		if (node->content) {
+			smart_str_appends(&str, (const char *) node->content);
+		}
 		node = node->next;
 	}
 
-	if (wholetext != NULL) {
-		ZVAL_STRING(retval, (char *) wholetext);
-		xmlFree(wholetext);
-	} else {
-		ZVAL_EMPTY_STRING(retval);
-	}
+	ZVAL_STR(retval, smart_str_extract(&str));
 
 	return SUCCESS;
 }
@@ -100,12 +91,12 @@ int dom_text_whole_text_read(dom_object *obj, zval *retval)
 /* }}} */
 
 /* {{{ URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#core-ID-38853C1D
+Modern spec URL: https://dom.spec.whatwg.org/#dom-text-splittext
 Since:
 */
 PHP_METHOD(DOMText, splitText)
 {
 	zval       *id;
-	xmlChar    *cur;
 	xmlChar    *first;
 	xmlChar    *second;
 	xmlNodePtr  node;
@@ -125,29 +116,19 @@ PHP_METHOD(DOMText, splitText)
 		RETURN_THROWS();
 	}
 
-	if (node->type != XML_TEXT_NODE && node->type != XML_CDATA_SECTION_NODE) {
-		/* TODO Add warning? */
-		RETURN_FALSE;
-	}
-
-	cur = xmlNodeGetContent(node);
-	if (cur == NULL) {
-		/* TODO Add warning? */
-		RETURN_FALSE;
-	}
+	const xmlChar *cur = php_dom_get_content_or_empty(node);
 	length = xmlUTF8Strlen(cur);
 
 	if (ZEND_LONG_INT_OVFL(offset) || (int)offset > length) {
-		/* TODO Add warning? */
-		xmlFree(cur);
+		if (php_dom_follow_spec_intern(intern)) {
+			php_dom_throw_error(INDEX_SIZE_ERR, /* strict */ true);
+		}
 		RETURN_FALSE;
 	}
 
 	first = xmlUTF8Strndup(cur, (int)offset);
 	second = xmlUTF8Strsub(cur, (int)offset, (int)(length - offset));
 
-	xmlFree(cur);
-
 	xmlNodeSetContent(node, first);
 	nnode = xmlNewDocText(node->doc, second);
 
@@ -155,8 +136,8 @@ PHP_METHOD(DOMText, splitText)
 	xmlFree(second);
 
 	if (nnode == NULL) {
-		/* TODO Add warning? */
-		RETURN_FALSE;
+		php_dom_throw_error(INVALID_STATE_ERR, /* strict */ true);
+		RETURN_THROWS();
 	}
 
 	if (node->parent != NULL) {
