@@ -244,8 +244,10 @@ gdImagePtr gdImageCreateFromGifCtx(gdIOCtxPtr fd) /* {{{ */
 					BitSet(buf[8], INTERLACE), &ZeroDataBlock);
 		} else {
 			if (!haveGlobalColormap) {
-				gdImageDestroy(im);
-				return 0;
+				// Still a valid gif, apply simple default palette as per spec
+				ColorMap[CM_RED][1] = 0xff;
+				ColorMap[CM_GREEN][1] = 0xff;
+				ColorMap[CM_BLUE][1] = 0xff;
 			}
 			ReadImage(im, fd, width, height,
 						ColorMap,
