@@ -5,62 +5,31 @@
 /** @not-serializable */
 class IntlDateFormatter
 {
-    /**
-     * @var int
-     * @cvalue UDAT_FULL
-     */
-    public const FULL = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UDAT_LONG
-     */
-    public const LONG = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UDAT_MEDIUM
-     */
-    public const MEDIUM = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UDAT_SHORT
-     */
-    public const SHORT = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UDAT_NONE
-     */
-    public const NONE = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UDAT_FULL_RELATIVE
-     */
-    public const RELATIVE_FULL = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UDAT_LONG_RELATIVE
-     */
-    public const RELATIVE_LONG = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UDAT_MEDIUM_RELATIVE
-     */
-    public const RELATIVE_MEDIUM = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UDAT_SHORT_RELATIVE
-     */
-    public const RELATIVE_SHORT = UNKNOWN;
+    /** @cvalue UDAT_FULL */
+    public const int FULL = UNKNOWN;
+    /** @cvalue UDAT_LONG */
+    public const int LONG = UNKNOWN;
+    /** @cvalue UDAT_MEDIUM */
+    public const int MEDIUM = UNKNOWN;
+    /** @cvalue UDAT_SHORT */
+    public const int SHORT = UNKNOWN;
+    /** @cvalue UDAT_NONE */
+    public const int NONE = UNKNOWN;
+    /** @cvalue UDAT_FULL_RELATIVE */
+    public const int RELATIVE_FULL = UNKNOWN;
+    /** @cvalue UDAT_LONG_RELATIVE */
+    public const int RELATIVE_LONG = UNKNOWN;
+    /** @cvalue UDAT_MEDIUM_RELATIVE */
+    public const int RELATIVE_MEDIUM = UNKNOWN;
+    /** @cvalue UDAT_SHORT_RELATIVE */
+    public const int RELATIVE_SHORT = UNKNOWN;
+    /** @cvalue UDAT_PATTERN */
+    public const int PATTERN = UNKNOWN;
 
-    /**
-     * @var int
-     * @cvalue UCAL_GREGORIAN
-     */
-    public const GREGORIAN = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UCAL_TRADITIONAL
-     */
-    public const TRADITIONAL = UNKNOWN;
+    /** @cvalue UCAL_GREGORIAN */
+    public const int GREGORIAN = UNKNOWN;
+    /** @cvalue UCAL_TRADITIONAL */
+    public const int TRADITIONAL = UNKNOWN;
 
     /**
      * @param IntlTimeZone|DateTimeZone|string|null $timezone
@@ -190,6 +159,11 @@ class IntlDateFormatter
      */
     public function parse(string $string, &$offset = null): int|float|false {}
 
+    /**
+     * @param int $offset
+     */
+    public function parseToCalendar(string $string, &$offset = null): int|float|false {}
+
     /**
      * @param int $offset
      * @return array<string, int>|false
