@@ -1,11 +1,9 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -19,9 +17,6 @@
 
 #include <php.h>
 
-PHP_FUNCTION( numfmt_format );
-PHP_FUNCTION( numfmt_format_currency );
-
 #define FORMAT_TYPE_DEFAULT	0
 #define FORMAT_TYPE_INT32	1
 #define FORMAT_TYPE_INT64	2
