@@ -6,7 +6,7 @@ const mbfl_language mbfl_language_uni = {
 	mbfl_no_language_uni,
 	"uni",
 	"uni",
-	(const char *(*)[])&mbfl_language_uni_aliases,
+	mbfl_language_uni_aliases,
 	mbfl_no_encoding_utf8,
 	mbfl_no_encoding_base64,
 	mbfl_no_encoding_base64
