@@ -77,6 +77,51 @@ void ir_reset_cfg(ir_ctx *ctx)
 	}
 }
 
+static uint32_t IR_NEVER_INLINE ir_cfg_remove_dead_inputs(ir_ctx *ctx, uint32_t *_blocks, ir_block *blocks, uint32_t bb_count)
+{
+	uint32_t b, count = 0;
+	ir_block *bb = blocks + 1;
+	ir_insn *insn;
+	ir_ref i, j, n, *ops, input;
+
+	for (b = 1; b <= bb_count; b++, bb++) {
+		bb->successors = count;
+		count += ctx->use_lists[bb->end].count;
+		bb->successors_count = 0;
+		bb->predecessors = count;
+		insn = &ctx->ir_base[bb->start];
+		if (insn->op == IR_MERGE || insn->op == IR_LOOP_BEGIN) {
+			n = insn->inputs_count;
+			ops = insn->ops;
+			for (i = 1, j = 1; i <= n; i++) {
+				input = ops[i];
+				if (_blocks[input]) {
+					if (i != j) {
+						ops[j] = ops[i];
+					}
+					j++;
+				} else if (input > 0) {
+					ir_use_list_remove_one(ctx, input, bb->start);
+				}
+			}
+			j--;
+			if (j != n) {
+				if (j == 1) {
+					insn->op = IR_BEGIN;
+				}
+				insn->inputs_count = j;
+				bb->predecessors_count = j;
+				j++;
+				for (;j <= n; j++) {
+					ops[j] = IR_UNUSED;
+				}
+			}
+		}
+		count += bb->predecessors_count;
+	}
+	return count;
+}
+
 int ir_build_cfg(ir_ctx *ctx)
 {
 	ir_ref n, *p, ref, start, end;
@@ -239,7 +284,10 @@ int ir_build_cfg(ir_ctx *ctx)
 		bb++;
 	} IR_BITSET_FOREACH_END();
 	bb_count = b - 1;
-	IR_ASSERT(count == edges_count * 2);
+	if (UNEXPECTED(count != edges_count * 2)) {
+		count = ir_cfg_remove_dead_inputs(ctx, _blocks, blocks, bb_count);
+		IR_ASSERT(count != edges_count * 2);
+	}
 	ir_mem_free(bb_starts);
 
 	/* Create an array of successor/predecessors control edges */
@@ -328,7 +376,7 @@ static void ir_remove_predecessor(ir_ctx *ctx, ir_block *bb, uint32_t from)
 
 static void ir_remove_merge_input(ir_ctx *ctx, ir_ref merge, ir_ref from)
 {
-	ir_ref i, j, n, k, *p, use;
+	ir_ref i, j, n, k, *p, *q, use;
 	ir_insn *use_insn;
 	ir_use_list *use_list;
 	ir_bitset life_inputs;
@@ -359,7 +407,7 @@ static void ir_remove_merge_input(ir_ctx *ctx, ir_ref merge, ir_ref from)
 		use_list = &ctx->use_lists[merge];
 		if (use_list->count > 1) {
 			n++;
-			for (k = 0, p = &ctx->use_edges[use_list->refs]; k < use_list->count; k++, p++) {
+			for (k = use_list->count, p = q = &ctx->use_edges[use_list->refs]; k > 0; p++, k--) {
 				use = *p;
 				use_insn = &ctx->ir_base[use];
 				if (use_insn->op == IR_PHI) {
@@ -379,8 +427,22 @@ static void ir_remove_merge_input(ir_ctx *ctx, ir_ref merge, ir_ref from)
 					for (j = 2; j <= n; j++) {
 						ir_insn_set_op(use_insn, j, IR_UNUSED);
 					}
-					ir_use_list_remove_all(ctx, merge, use);
+					continue;
 				}
+
+				/*compact use list */
+				if (p != q){
+					*q = use;
+				}
+				q++;
+			}
+
+			if (p != q) {
+				use_list->count -= (p - q);
+				do {
+					*q = IR_UNUSED; /* clenu-op the removed tail */
+					q++;
+				} while (p != q);
 			}
 		}
 	} else {
@@ -389,7 +451,7 @@ static void ir_remove_merge_input(ir_ctx *ctx, ir_ref merge, ir_ref from)
 		use_list = &ctx->use_lists[merge];
 		if (use_list->count > 1) {
 			n++;
-			for (k = 0, p = &ctx->use_edges[use_list->refs]; k < use_list->count; k++, p++) {
+			for (k = use_list->count, p = &ctx->use_edges[use_list->refs]; k > 0; p++, k--) {
 				use = *p;
 				use_insn = &ctx->ir_base[use];
 				if (use_insn->op == IR_PHI) {
