@@ -1439,7 +1439,8 @@ static SSL_CTX *php_openssl_create_sni_server_ctx(char *cert_path, char *key_pat
 }
 /* }}} */
 
-static zend_result php_openssl_enable_server_sni(php_stream *stream, php_openssl_netstream_data_t *sslsock)  /* {{{ */
+static zend_result php_openssl_enable_server_sni(
+		php_stream *stream, php_openssl_netstream_data_t *sslsock, bool verify_peer)
 {
 	zval *val;
 	zval *current;
@@ -1560,6 +1561,12 @@ static zend_result php_openssl_enable_server_sni(php_stream *stream, php_openssl
 			return FAILURE;
 		}
 
+		if (!verify_peer) {
+			php_openssl_disable_peer_verification(ctx, stream);
+		} else if (FAILURE == php_openssl_enable_peer_verification(ctx, stream)) {
+			return FAILURE;
+		}
+
 		sslsock->sni_certs[i].name = pestrdup(ZSTR_VAL(key), php_stream_is_persistent(stream));
 		sslsock->sni_certs[i].ctx = ctx;
 		++i;
@@ -1570,7 +1577,6 @@ static zend_result php_openssl_enable_server_sni(php_stream *stream, php_openssl
 
 	return SUCCESS;
 }
-/* }}} */
 
 static void php_openssl_enable_client_sni(php_stream *stream, php_openssl_netstream_data_t *sslsock) /* {{{ */
 {
@@ -1662,6 +1668,7 @@ static zend_result php_openssl_setup_crypto(php_stream *stream,
 	char *cipherlist = NULL;
 	char *alpn_protocols = NULL;
 	zval *val;
+	bool verify_peer = false;
 
 	if (sslsock->ssl_handle) {
 		if (sslsock->s.is_blocked) {
@@ -1713,8 +1720,11 @@ static zend_result php_openssl_setup_crypto(php_stream *stream,
 
 	if (GET_VER_OPT("verify_peer") && !zend_is_true(val)) {
 		php_openssl_disable_peer_verification(sslsock->ctx, stream);
-	} else if (FAILURE == php_openssl_enable_peer_verification(sslsock->ctx, stream)) {
-		return FAILURE;
+	} else {
+		verify_peer = true;
+		if (FAILURE == php_openssl_enable_peer_verification(sslsock->ctx, stream)) {
+			return FAILURE;
+		}
 	}
 
 	/* callback for the passphrase (for localcert) */
@@ -1815,7 +1825,7 @@ static zend_result php_openssl_setup_crypto(php_stream *stream,
 
 #ifdef HAVE_TLS_SNI
 	/* Enable server-side SNI */
-	if (!sslsock->is_client && php_openssl_enable_server_sni(stream, sslsock) == FAILURE) {
+	if (!sslsock->is_client && php_openssl_enable_server_sni(stream, sslsock, verify_peer) == FAILURE) {
 		return FAILURE;
 	}
 #endif
@@ -1897,6 +1907,15 @@ static int php_openssl_capture_peer_certs(php_stream *stream,
 }
 /* }}} */
 
+static zend_result php_openssl_set_blocking(php_openssl_netstream_data_t *sslsock, int block)
+{
+	zend_result result = php_set_sock_blocking(sslsock->s.socket, block);
+	if (EXPECTED(SUCCESS == result)) {
+		sslsock->s.is_blocked = block;
+	}
+	return result;
+}
+
 static int php_openssl_enable_crypto(php_stream *stream,
 		php_openssl_netstream_data_t *sslsock,
 		php_stream_xport_crypto_param *cparam) /* {{{ */
@@ -1925,8 +1944,7 @@ static int php_openssl_enable_crypto(php_stream *stream,
 			sslsock->state_set = 1;
 		}
 
-		if (SUCCESS == php_set_sock_blocking(sslsock->s.socket, 0)) {
-			sslsock->s.is_blocked = 0;
+		if (SUCCESS == php_openssl_set_blocking(sslsock, 0)) {
 			/* The following mode are added only if we are able to change socket
 			 * to non blocking mode which is also used for read and write */
 			SSL_set_mode(sslsock->ssl_handle, SSL_MODE_ENABLE_PARTIAL_WRITE | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
@@ -1979,8 +1997,8 @@ static int php_openssl_enable_crypto(php_stream *stream,
 			}
 		} while (retry);
 
-		if (sslsock->s.is_blocked != blocked && SUCCESS == php_set_sock_blocking(sslsock->s.socket, blocked)) {
-			sslsock->s.is_blocked = blocked;
+		if (sslsock->s.is_blocked != blocked) {
+			php_openssl_set_blocking(sslsock, blocked);
 		}
 
 		if (n == 1) {
@@ -2063,8 +2081,8 @@ static ssize_t php_openssl_sockop_io(int read, php_stream *stream, char *buf, si
 			timeout = &sslsock->s.timeout;
 		}
 
-		if (timeout && php_set_sock_blocking(sslsock->s.socket, 0) == SUCCESS) {
-			sslsock->s.is_blocked = 0;
+		if (timeout) {
+			php_openssl_set_blocking(sslsock, 0);
 		}
 
 		if (!sslsock->s.is_blocked && timeout && (timeout->tv_sec > 0 || (timeout->tv_sec == 0 && timeout->tv_usec))) {
@@ -2088,8 +2106,7 @@ static ssize_t php_openssl_sockop_io(int read, php_stream *stream, char *buf, si
 				if (php_openssl_compare_timeval(elapsed_time, *timeout) > 0 ) {
 					/* If the socket was originally blocking, set it back. */
 					if (began_blocked) {
-						php_set_sock_blocking(sslsock->s.socket, 1);
-						sslsock->s.is_blocked = 1;
+						php_openssl_set_blocking(sslsock, 1);
 					}
 					sslsock->s.timeout_event = 1;
 					return -1;
@@ -2184,8 +2201,8 @@ static ssize_t php_openssl_sockop_io(int read, php_stream *stream, char *buf, si
 		}
 
 		/* And if we were originally supposed to be blocking, let's reset the socket to that. */
-		if (began_blocked && php_set_sock_blocking(sslsock->s.socket, 1) == SUCCESS) {
-			sslsock->s.is_blocked = 1;
+		if (began_blocked) {
+			php_openssl_set_blocking(sslsock, 1);
 		}
 
 		return 0 > nr_bytes ? 0 : nr_bytes;
@@ -2491,8 +2508,8 @@ static int php_openssl_sockop_set_option(php_stream *stream, int option, int val
 							timeout = &tv;
 						}
 
-						if (timeout && php_set_sock_blocking(sslsock->s.socket, 0) == SUCCESS) {
-							sslsock->s.is_blocked = 0;
+						if (timeout) {
+							php_openssl_set_blocking(sslsock, 0);
 						}
 
 						if (!sslsock->s.is_blocked && timeout && (timeout->tv_sec > 0 || (timeout->tv_sec == 0 && timeout->tv_usec))) {
@@ -2516,8 +2533,7 @@ static int php_openssl_sockop_set_option(php_stream *stream, int option, int val
 								if (php_openssl_compare_timeval(elapsed_time, *timeout) > 0 ) {
 									/* If the socket was originally blocking, set it back. */
 									if (began_blocked) {
-										php_set_sock_blocking(sslsock->s.socket, 1);
-										sslsock->s.is_blocked = 1;
+										php_openssl_set_blocking(sslsock, 1);
 									}
 									sslsock->s.timeout_event = 1;
 									return PHP_STREAM_OPTION_RETURN_ERR;
@@ -2568,8 +2584,7 @@ static int php_openssl_sockop_set_option(php_stream *stream, int option, int val
 
 						if (began_blocked && !sslsock->s.is_blocked) {
 							// Set it back to blocking
-							php_set_sock_blocking(sslsock->s.socket, 1);
-							sslsock->s.is_blocked = 1;
+							php_openssl_set_blocking(sslsock, 1);
 						}
 					} else {
 #ifdef PHP_WIN32
