@@ -1,13 +1,11 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
-  +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2016 The PHP Group                                |
+  | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
+  | https://www.php.net/license/3_01.txt                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -16,16 +14,17 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #ifndef PHP_PDO_H
 #define PHP_PDO_H
 
 #include "zend.h"
 
-extern zend_module_entry pdo_module_entry;
+PHPAPI extern zend_module_entry pdo_module_entry;
 #define phpext_pdo_ptr &pdo_module_entry
 
+PHPAPI extern zend_class_entry *pdo_dbh_ce;
+PHPAPI extern zend_object *pdo_dbh_new(zend_class_entry *ce);
+
 #include "php_version.h"
 #define PHP_PDO_VERSION PHP_VERSION
 
@@ -51,37 +50,29 @@ PHP_MINIT_FUNCTION(pdo);
 PHP_MSHUTDOWN_FUNCTION(pdo);
 PHP_MINFO_FUNCTION(pdo);
 
-ZEND_BEGIN_MODULE_GLOBALS(pdo)
-	zend_long  global_value;
-ZEND_END_MODULE_GLOBALS(pdo)
-
-#ifdef ZTS
-# define PDOG(v) TSRMG(pdo_globals_id, zend_pdo_globals *, v)
-#else
-# define PDOG(v) (pdo_globals.v)
-#endif
-
 #define REGISTER_PDO_CLASS_CONST_LONG(const_name, value) \
 	zend_declare_class_constant_long(php_pdo_get_dbh_ce(), const_name, sizeof(const_name)-1, (zend_long)value);
 
-#define REGISTER_PDO_CLASS_CONST_STRING(const_name, value) \
-	zend_declare_class_constant_stringl(php_pdo_get_dbh_ce(), const_name, sizeof(const_name)-1, value, sizeof(value)-1);
+#define LONG_CONST(c) (zend_long) c
 
-#define PDO_CONSTRUCT_CHECK	\
-	if (!dbh->driver) {	\
-		pdo_raise_impl_error(dbh, NULL, "00000", "PDO constructor was not called");	\
-		return;	\
-	}	\
+#define PDO_CONSTRUCT_CHECK_COND dbh->driver
+#define PDO_CONSTRUCT_CHECK_FAIL() \
+		{ \
+			zend_throw_error(NULL, "%s object is uninitialized", ZSTR_VAL(Z_OBJ(EX(This))->ce->name)); \
+		} \
+
+#define PDO_CONSTRUCT_CHECK \
+	if (!(PDO_CONSTRUCT_CHECK_COND)) { \
+		PDO_CONSTRUCT_CHECK_FAIL(); \
+		RETURN_THROWS(); \
+	} \
+
+
+#define PDO_CONSTRUCT_CHECK_WITH_CLEANUP(cleanup) \
+	if (!(PDO_CONSTRUCT_CHECK_COND)) { \
+		PDO_CONSTRUCT_CHECK_FAIL(); \
+		goto cleanup; \
+	} \
 
 
 #endif	/* PHP_PDO_H */
-
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
