@@ -20,8 +20,6 @@
 
 #ifdef HAVE_PGSQL
 
-#define PHP_PGSQL_API_VERSION 20140217
-
 extern zend_module_entry pgsql_module_entry;
 #define pgsql_module_ptr &pgsql_module_entry
 
@@ -48,10 +46,6 @@ extern zend_module_entry pgsql_module_entry;
 # endif
 #endif
 
-#ifdef HAVE_PGSQL_WITH_MULTIBYTE_SUPPORT
-const char * pg_encoding_to_char(int encoding);
-#endif
-
 PHP_MINIT_FUNCTION(pgsql);
 PHP_MSHUTDOWN_FUNCTION(pgsql);
 PHP_RINIT_FUNCTION(pgsql);
@@ -181,14 +175,17 @@ static const php_stream_ops php_stream_pgsql_fd_ops = {
 	php_pgsql_fd_set_option
 };
 
+#define PGSQL_MAX_REGEXES 11
+
 ZEND_BEGIN_MODULE_GLOBALS(pgsql)
 	zend_long num_links,num_persistent;
 	zend_long max_links,max_persistent;
 	bool allow_persistent;
-    int ignore_notices;
+	int ignore_notices;
 	zend_long auto_reset_persistent;
 	int log_notices;
 	zend_object *default_link; /* default link when connection is omitted */
+	zend_string *regexes[PGSQL_MAX_REGEXES];
 	HashTable field_oids;
 	HashTable table_oids;
 	HashTable connections;
