@@ -1612,10 +1612,16 @@ function pos(array|object $array): mixed {}
 
 function key(array|object $array): int|string|null {}
 
-/** @compile-time-eval */
+/**
+ * @compile-time-eval
+ * @frameless-function {"arity": 2}
+ */
 function min(mixed $value, mixed ...$values): mixed {}
 
-/** @compile-time-eval */
+/**
+ * @compile-time-eval
+ * @frameless-function {"arity": 2}
+ */
 function max(mixed $value, mixed ...$values): mixed {}
 
 function array_walk(array|object &$array, callable $callback, mixed $arg = UNKNOWN): true {}
@@ -1624,6 +1630,8 @@ function array_walk_recursive(array|object &$array, callable $callback, mixed $a
 
 /**
  * @compile-time-eval
+ * @frameless-function {"arity": 2}
+ * @frameless-function {"arity": 3}
  */
 function in_array(mixed $needle, array $haystack, bool $strict = false): bool {}
 
@@ -1862,6 +1870,14 @@ function array_reduce(array $array, callable $callback, mixed $initial = null):
 
 function array_filter(array $array, ?callable $callback = null, int $mode = 0): array {}
 
+function array_find(array $array, callable $callback): mixed {}
+
+function array_find_key(array $array, callable $callback): mixed {}
+
+function array_any(array $array, callable $callback): bool {}
+
+function array_all(array $array, callable $callback): bool {}
+
 function array_map(?callable $callback, array $array, array ...$arrays): array {}
 
 /**
@@ -1910,7 +1926,7 @@ function constant(string $name): mixed {}
 function ip2long(string $ip): int|false {}
 
 /** @refcount 1 */
-function long2ip(int $ip): string|false {}
+function long2ip(int $ip): string {}
 
 /**
  * @return string|array<string, string>|false
@@ -1981,7 +1997,7 @@ function show_source(string $filename, bool $return = false): string|bool {}
 function php_strip_whitespace(string $filename): string {}
 
 /** @refcount 1 */
-function highlight_string(string $string, bool $return = false): string|bool {}
+function highlight_string(string $string, bool $return = false): string|true {}
 
 function ini_get(string $option): string|false {}
 
@@ -2007,7 +2023,7 @@ function set_include_path(string $include_path): string|false {}
 function get_include_path(): string|false {}
 
 /** @refcount 1 */
-function print_r(mixed $value, bool $return = false): string|bool {}
+function print_r(mixed $value, bool $return = false): string|true {}
 
 function connection_aborted(): int {}
 
@@ -2093,9 +2109,9 @@ function crypt(#[\SensitiveParameter] string $string, string $salt): string {}
 #ifdef HAVE_STRPTIME
 /**
  * @return array<string, int|string>|false
- * @deprecated
  * @refcount 1
  */
+#[\Deprecated(since: '8.2', message: 'use date_parse_from_format() (for locale-independent parsing), or IntlDateFormatter::parse() (for locale-dependent parsing) instead')]
 function strptime(string $timestamp, string $format): array|false {}
 #endif
 
@@ -2148,7 +2164,7 @@ function getmxrr(string $hostname, &$hosts, &$weights = null): bool {}
 
 /* net.c */
 
-#if (defined(PHP_WIN32) || HAVE_GETIFADDRS || defined(__PASE__))
+#if (defined(PHP_WIN32) || defined(HAVE_GETIFADDRS) || defined(__PASE__))
 function net_get_interfaces(): array|false {}
 #endif
 
@@ -2197,18 +2213,14 @@ function openlog(string $prefix, int $flags, int $facility): true {}
 
 function closelog(): true {}
 
-function syslog(int $priority, string $message): true {} // TODO make return type void
+function syslog(int $priority, string $message): true {}
 #endif
 
-#ifdef HAVE_INET_NTOP
 /** @refcount 1 */
 function inet_ntop(string $ip): string|false {}
-#endif
 
-#ifdef HAVE_INET_PTON
 /** @refcount 1 */
 function inet_pton(string $ip): string|false {}
-#endif
 
 /* metaphone.c */
 
@@ -2262,7 +2274,7 @@ function get_html_translation_table(int $table = HTML_SPECIALCHARS, int $flags =
 
 function assert(mixed $assertion, Throwable|string|null $description = null): bool {}
 
-/** @deprecated */
+#[\Deprecated(since: '8.3')]
 function assert_options(int $option, mixed $value = UNKNOWN): mixed {}
 
 /* string.c */
@@ -2296,7 +2308,11 @@ function nl_langinfo(int $item): string|false {}
 
 function strcoll(string $string1, string $string2): int {}
 
-/** @compile-time-eval */
+/**
+ * @compile-time-eval
+ * @frameless-function {"arity": 1}
+ * @frameless-function {"arity": 2}
+ */
 function trim(string $string, string $characters = " \n\r\t\v\0"): string {}
 
 /** @compile-time-eval */
@@ -2323,6 +2339,8 @@ function explode(string $separator, string $string, int $limit = PHP_INT_MAX): a
 
 /**
  * @compile-time-eval
+ * @frameless-function {"arity": 1}
+ * @frameless-function {"arity": 2}
  */
 function implode(string|array $separator, ?array $array = null): string {}
 
@@ -2347,7 +2365,11 @@ function str_decrement(string $string): string {}
 /** @refcount 1 */
 function basename(string $path, string $suffix = ""): string {}
 
-/** @refcount 1 */
+/**
+ * @refcount 1
+ * @frameless-function {"arity": 1}
+ * @frameless-function {"arity": 2}
+ */
 function dirname(string $path, int $levels = 1): string {}
 
 /**
@@ -2365,13 +2387,19 @@ function stristr(string $haystack, string $needle, bool $before_needle = false):
 /**
  * @compile-time-eval
  * @refcount 1
+ * @frameless-function {"arity": 2}
+ * @frameless-function {"arity": 3}
  */
 function strstr(string $haystack, string $needle, bool $before_needle = false): string|false {}
 
 /** @alias strstr */
 function strchr(string $haystack, string $needle, bool $before_needle = false): string|false {}
 
-/** @compile-time-eval */
+/**
+ * @compile-time-eval
+ * @frameless-function {"arity": 2}
+ * @frameless-function {"arity": 3}
+ */
 function strpos(string $haystack, string $needle, int $offset = 0): int|false {}
 
 /** @compile-time-eval */
@@ -2389,10 +2417,16 @@ function strripos(string $haystack, string $needle, int $offset = 0): int|false
  */
 function strrchr(string $haystack, string $needle, bool $before_needle = false): string|false {}
 
-/** @compile-time-eval */
+/**
+ * @compile-time-eval
+ * @frameless-function {"arity": 2}
+ */
 function str_contains(string $haystack, string $needle): bool {}
 
-/** @compile-time-eval */
+/**
+ * @compile-time-eval
+ * @frameless-function {"arity": 2}
+ */
 function str_starts_with(string $haystack, string $needle): bool {}
 
 /** @compile-time-eval */
@@ -2404,7 +2438,11 @@ function str_ends_with(string $haystack, string $needle): bool {}
  */
 function chunk_split(string $string, int $length = 76, string $separator = "\r\n"): string {}
 
-/** @compile-time-eval */
+/**
+ * @compile-time-eval
+ * @frameless-function {"arity": 2}
+ * @frameless-function {"arity": 3}
+ */
 function substr(string $string, int $offset, ?int $length = null): string {}
 
 /**
@@ -2442,6 +2480,8 @@ function ucwords(string $string, string $separators = " \t\r\n\f\v"): string {}
 
 /**
  * @compile-time-eval
+ * @frameless-function {"arity": 2}
+ * @frameless-function {"arity": 3}
  */
 function strtr(string $string, string|array $from, ?string $to = null): string {}
 
@@ -2480,6 +2520,7 @@ function stripslashes(string $string): string {}
  * @param int $count
  * @return string|array<int|string, string>
  * @compile-time-eval
+ * @frameless-function {"arity": 3}
  */
 function str_replace(array|string $search, array|string $replace, string|array $subject, &$count = null): string|array {}
 
@@ -2587,15 +2628,15 @@ function substr_compare(string $haystack, string $needle, int $offset, ?int $len
 /**
  * @compile-time-eval
  * @refcount 1
- * @deprecated
  */
+#[\Deprecated(since: '8.2', message: 'visit the php.net documentation for various alternatives')]
 function utf8_encode(string $string): string {}
 
 /**
  * @compile-time-eval
  * @refcount 1
- * @deprecated
  */
+#[\Deprecated(since: '8.2', message: 'visit the php.net documentation for various alternatives')]
 function utf8_decode(string $string): string {}
 
 /* dir.c */
@@ -2965,6 +3006,17 @@ function pfsockopen(string $hostname, int $port = -1, &$error_code = null, &$err
 /** @refcount 1 */
 function http_build_query(array|object $data, string $numeric_prefix = "", ?string $arg_separator = null, int $encoding_type = PHP_QUERY_RFC1738): string {}
 
+function http_get_last_response_headers(): ?array {}
+
+function http_clear_last_response_headers(): void {}
+
+/**
+ * @param array|null $options
+ * @return array<int, array>
+ * @refcount 1
+ */
+function request_parse_body(?array $options = null): array {}
+
 /* image.c */
 
 /**
@@ -3066,8 +3118,19 @@ function ceil(int|float $num): float {}
 /** @compile-time-eval */
 function floor(int|float $num): float {}
 
+enum RoundingMode {
+    case HalfAwayFromZero;
+    case HalfTowardsZero;
+    case HalfEven;
+    case HalfOdd;
+    case TowardsZero;
+    case AwayFromZero;
+    case NegativeInfinity;
+    case PositiveInfinity;
+}
+
 /** @compile-time-eval */
-function round(int|float $num, int $precision = 0, int $mode = PHP_ROUND_HALF_UP): float {}
+function round(int|float $num, int $precision = 0, int|RoundingMode $mode = RoundingMode::HalfAwayFromZero): float {}
 
 /** @compile-time-eval */
 function sin(float $num): float {}
@@ -3177,6 +3240,7 @@ function decoct(int $num): string {}
 /**
  * @compile-time-eval
  * @refcount 1
+ * @frameless-function {"arity": 1}
  */
 function dechex(int $num): string {}
 
@@ -3202,6 +3266,11 @@ function fmod(float $num1, float $num2): float {}
  */
 function fdiv(float $num1, float $num2): float {}
 
+/**
+ * @compile-time-eval
+ */
+function fpow(float $num, float $exponent): float {}
+
 /* microtime.c */
 
 #ifdef HAVE_GETTIMEOFDAY
@@ -3314,7 +3383,7 @@ function stream_select(?array &$read, ?array &$write, ?array &$except, ?int $sec
 function stream_context_create(?array $options = null, ?array $params = null) {}
 
 /** @param resource $context */
-function stream_context_set_params($context, array $params): bool {}
+function stream_context_set_params($context, array $params): true {}
 
 /**
  * @param resource $context
@@ -3324,10 +3393,10 @@ function stream_context_set_params($context, array $params): bool {}
 function stream_context_get_params($context): array {}
 
 /** @param resource $context */
-function stream_context_set_option($context, array|string $wrapper_or_options, ?string $option_name = null, mixed $value = UNKNOWN): bool {}
+function stream_context_set_option($context, array|string $wrapper_or_options, ?string $option_name = null, mixed $value = UNKNOWN): true {}
 
 /** @param resource $context */
-function stream_context_set_options($context, array $options): bool {}
+function stream_context_set_options($context, array $options): true {}
 
 /**
  * @param resource $stream_or_context
@@ -3582,6 +3651,7 @@ function is_double(mixed $value): bool {}
 
 /**
  * @compile-time-eval
+ * @frameless-function {"arity": 1}
  */
 function is_numeric(mixed $value): bool {}
 
@@ -3671,19 +3741,19 @@ function get_headers(string $url, bool $associative = false, $context = null): a
  * @param resource $brigade
  * @refcount 1
  */
-function stream_bucket_make_writeable($brigade): ?object {}
+function stream_bucket_make_writeable($brigade): ?StreamBucket {}
 
 /** @param resource $brigade */
-function stream_bucket_prepend($brigade, object $bucket): void {}
+function stream_bucket_prepend($brigade, StreamBucket $bucket): void {}
 
 /** @param resource $brigade */
-function stream_bucket_append($brigade, object $bucket): void {}
+function stream_bucket_append($brigade, StreamBucket $bucket): void {}
 
 /**
  * @param resource $stream
  * @refcount 1
  */
-function stream_bucket_new($stream, string $buffer): object {}
+function stream_bucket_new($stream, string $buffer): StreamBucket {}
 
 /**
  * @return array<int, string>
