@@ -16,7 +16,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include "php.h"
@@ -32,18 +32,13 @@
 #include "php_sysvsem.h"
 #include "ext/standard/info.h"
 
-#if !HAVE_SEMUN
-
+#ifndef HAVE_UNION_SEMUN
 union semun {
 	int val;                    /* value for SETVAL */
 	struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
 	unsigned short int *array;  /* array for GETALL, SETALL */
 	struct seminfo *__buf;      /* buffer for IPC_INFO */
 };
-
-#undef HAVE_SEMUN
-#define HAVE_SEMUN 1
-
 #endif
 
 /* {{{ sysvsem_module_entry */
@@ -173,12 +168,6 @@ PHP_MINFO_FUNCTION(sysvsem)
 }
 /* }}} */
 
-#define SETVAL_WANTS_PTR
-
-#if defined(_AIX)
-#undef SETVAL_WANTS_PTR
-#endif
-
 /* {{{ Return an id for the semaphore with the given key, and allow max_acquire (default 1) processes to acquire it simultaneously */
 PHP_FUNCTION(sem_get)
 {
@@ -247,24 +236,11 @@ PHP_FUNCTION(sem_get)
 	/* If we are the only user, then take this opportunity to set the max. */
 
 	if (count == 1) {
-#if HAVE_SEMUN
-		/* This is correct for Linux which has union semun. */
 		union semun semarg;
 		semarg.val = max_acquire;
 		if (semctl(semid, SYSVSEM_SEM, SETVAL, semarg) == -1) {
 			php_error_docref(NULL, E_WARNING, "Failed for key 0x" ZEND_XLONG_FMT ": %s", key, strerror(errno));
 		}
-#elif defined(SETVAL_WANTS_PTR)
-		/* This is correct for Solaris 2.6 which does not have union semun. */
-		if (semctl(semid, SYSVSEM_SEM, SETVAL, &max_acquire) == -1) {
-			php_error_docref(NULL, E_WARNING, "Failed for key 0x%lx: %s", key, strerror(errno));
-		}
-#else
-		/* This works for i.e. AIX */
-		if (semctl(semid, SYSVSEM_SEM, SETVAL, max_acquire) == -1) {
-			php_error_docref(NULL, E_WARNING, "Failed for key 0x%lx: %s", key, strerror(errno));
-		}
-#endif
 	}
 
 	/* Set semaphore 1 back to zero. */
@@ -357,10 +333,8 @@ PHP_FUNCTION(sem_remove)
 {
 	zval *arg_id;
 	sysvsem_sem *sem_ptr;
-#if HAVE_SEMUN
 	union semun un;
 	struct semid_ds buf;
-#endif
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg_id, sysvsem_ce) == FAILURE) {
 		RETURN_THROWS();
@@ -368,21 +342,13 @@ PHP_FUNCTION(sem_remove)
 
 	sem_ptr = Z_SYSVSEM_P(arg_id);
 
-#if HAVE_SEMUN
 	un.buf = &buf;
 	if (semctl(sem_ptr->semid, 0, IPC_STAT, un) < 0) {
-#else
-	if (semctl(sem_ptr->semid, 0, IPC_STAT, NULL) < 0) {
-#endif
 		php_error_docref(NULL, E_WARNING, "SysV semaphore for key 0x%x does not (any longer) exist", sem_ptr->key);
 		RETURN_FALSE;
 	}
 
-#if HAVE_SEMUN
 	if (semctl(sem_ptr->semid, 0, IPC_RMID, un) < 0) {
-#else
-	if (semctl(sem_ptr->semid, 0, IPC_RMID, NULL) < 0) {
-#endif
 		php_error_docref(NULL, E_WARNING, "Failed for SysV semaphore for key 0x%x: %s", sem_ptr->key, strerror(errno));
 		RETURN_FALSE;
 	}
