@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -16,12 +14,10 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #ifndef PHP_SYSVSHM_H
 #define PHP_SYSVSHM_H
 
-#if HAVE_SYSVSHM
+#ifdef HAVE_SYSVSHM
 
 extern zend_module_entry sysvshm_module_entry;
 #define sysvshm_module_ptr &sysvshm_module_entry
@@ -34,18 +30,12 @@ extern zend_module_entry sysvshm_module_entry;
 #ifdef PHP_WIN32
 # include <TSRM/tsrm_win32.h>
 # include "win32/ipc.h"
-# ifndef THREAD_LS
-#  define THREAD_LS
-# endif
 #else
 # include <sys/ipc.h>
 # include <sys/shm.h>
 #endif
 
-#define PHP_SHM_RSRC_NAME "sysvshm"
-
 typedef struct {
-	int le_shm;
 	zend_long init_mem;
 } sysvshm_module;
 
@@ -68,16 +58,11 @@ typedef struct {
 	key_t key;               /* key set by user */
 	zend_long id;                 /* returned by shmget */
 	sysvshm_chunk_head *ptr; /* memory address of shared memory */
+	zend_object std;
 } sysvshm_shm;
 
 PHP_MINIT_FUNCTION(sysvshm);
-PHP_FUNCTION(shm_attach);
-PHP_FUNCTION(shm_detach);
-PHP_FUNCTION(shm_remove);
-PHP_FUNCTION(shm_put_var);
-PHP_FUNCTION(shm_get_var);
-PHP_FUNCTION(shm_has_var);
-PHP_FUNCTION(shm_remove_var);
+PHP_MINFO_FUNCTION(sysvshm);
 
 extern sysvshm_module php_sysvshm;
 
