@@ -15,7 +15,7 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include "php.h"
@@ -40,6 +40,8 @@
 
 #include "tidy_arginfo.h"
 
+#include "Zend/zend_exceptions.h"
+
 /* compatibility with older versions of libtidy */
 #ifndef TIDY_CALL
 #define TIDY_CALL
@@ -54,7 +56,7 @@
 #define TIDY_FETCH_OBJECT	\
 	PHPTidyObj *obj;	\
 	zval *object; \
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O", &object, tidy_ce_doc) == FAILURE) {	\
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O", &object, tidy_ce_doc) != SUCCESS) {	\
 		RETURN_THROWS();	\
 	}	\
 	obj = Z_TIDY_P(object);	\
@@ -69,7 +71,7 @@
 #define TIDY_FETCH_ONLY_OBJECT	\
 	PHPTidyObj *obj;	\
 	TIDY_SET_CONTEXT; \
-	if (zend_parse_parameters_none() == FAILURE) {	\
+	if (zend_parse_parameters_none() != SUCCESS) {	\
 		RETURN_THROWS();	\
 	}	\
 	obj = Z_TIDY_P(object);	\
@@ -118,7 +120,7 @@ static inline PHPTidyObj *php_tidy_fetch_object(zend_object *obj) {
 /* }}} */
 
 /* {{{ ext/tidy prototypes */
-static zend_string *php_tidy_file_to_mem(char *, bool);
+static zend_string *php_tidy_file_to_mem(const char *, bool);
 static void tidy_object_free_storage(zend_object *);
 static zend_object *tidy_object_new_node(zend_class_entry *);
 static zend_object *tidy_object_new_doc(zend_class_entry *);
@@ -129,12 +131,12 @@ static void tidy_doc_update_properties(PHPTidyObj *);
 static void tidy_add_node_default_properties(PHPTidyObj *);
 static void *php_tidy_get_opt_val(PHPTidyDoc *, TidyOption, TidyOptionType *);
 static void php_tidy_create_node(INTERNAL_FUNCTION_PARAMETERS, tidy_base_nodetypes);
-static int _php_tidy_set_tidy_opt(TidyDoc, char *, zval *);
-static int _php_tidy_apply_config_array(TidyDoc doc, HashTable *ht_options);
+static int _php_tidy_set_tidy_opt(TidyDoc, const char *, zval *);
+static int _php_tidy_apply_config_array(TidyDoc doc, const HashTable *ht_options);
 static PHP_INI_MH(php_tidy_set_clean_output);
 static void php_tidy_clean_output_start(const char *name, size_t name_len);
 static php_output_handler *php_tidy_output_handler_init(const char *handler_name, size_t handler_name_len, size_t chunk_size, int flags);
-static int php_tidy_output_handler(void **nothing, php_output_context *output_context);
+static zend_result php_tidy_output_handler(void **nothing, php_output_context *output_context);
 
 static PHP_MINIT_FUNCTION(tidy);
 static PHP_MSHUTDOWN_FUNCTION(tidy);
@@ -195,7 +197,7 @@ static void TIDY_CALL php_tidy_free(void *buf)
 
 static void TIDY_CALL php_tidy_panic(ctmbstr msg)
 {
-	php_error_docref(NULL, E_ERROR, "Could not allocate memory for tidy! (Reason: %s)", (char *)msg);
+	php_error_docref(NULL, E_ERROR, "Could not allocate memory for tidy! (Reason: %s)", (const char *)msg);
 }
 
 static void php_tidy_load_config(TidyDoc doc, const char *path)
@@ -208,7 +210,7 @@ static void php_tidy_load_config(TidyDoc doc, const char *path)
 	}
 }
 
-static zend_result php_tidy_apply_config(TidyDoc doc, zend_string *str_string, HashTable *ht_options)
+static zend_result php_tidy_apply_config(TidyDoc doc, const zend_string *str_string, const HashTable *ht_options)
 {
 	if (ht_options) {
 		return _php_tidy_apply_config_array(doc, ht_options);
@@ -221,7 +223,7 @@ static zend_result php_tidy_apply_config(TidyDoc doc, zend_string *str_string, H
 	return SUCCESS;
 }
 
-static int _php_tidy_set_tidy_opt(TidyDoc doc, char *optname, zval *value)
+static int _php_tidy_set_tidy_opt(TidyDoc doc, const char *optname, zval *value)
 {
 	TidyOption opt = tidyGetOptionByName(doc, optname);
 	zend_string *str, *tmp_str;
@@ -273,6 +275,17 @@ static int _php_tidy_set_tidy_opt(TidyDoc doc, char *optname, zval *value)
 	return FAILURE;
 }
 
+static void tidy_create_node_object(zval *zv, PHPTidyDoc *ptdoc, TidyNode node)
+{
+	tidy_instantiate(tidy_ce_node, zv);
+	PHPTidyObj *newobj = Z_TIDY_P(zv);
+	newobj->node = node;
+	newobj->type = is_node;
+	newobj->ptdoc = ptdoc;
+	newobj->ptdoc->ref_count++;
+	tidy_add_node_default_properties(newobj);
+}
+
 static void php_tidy_quick_repair(INTERNAL_FUNCTION_PARAMETERS, bool is_file)
 {
 	char *enc = NULL;
@@ -310,7 +323,7 @@ static void php_tidy_quick_repair(INTERNAL_FUNCTION_PARAMETERS, bool is_file)
 	if (ZEND_SIZE_T_UINT_OVFL(ZSTR_LEN(data))) {
 		if (is_file) {
 			zend_string_release_ex(data, false);
-			zend_argument_value_error(1, "Input string is too long");
+			zend_argument_value_error(1, "File content is too long");
 		} else {
 			zend_argument_value_error(1, "is too long");
 		}
@@ -358,7 +371,7 @@ static void php_tidy_quick_repair(INTERNAL_FUNCTION_PARAMETERS, bool is_file)
 
 				tidySaveBuffer (doc, &output);
 				FIX_BUFFER(&output);
-				RETVAL_STRINGL((char *) output.bp, output.size ? output.size-1 : 0);
+				RETVAL_STRINGL((const char *) output.bp, output.size ? output.size-1 : 0);
 				tidyBufFree(&output);
 			} else {
 				RETVAL_FALSE;
@@ -375,7 +388,7 @@ static void php_tidy_quick_repair(INTERNAL_FUNCTION_PARAMETERS, bool is_file)
 	tidyRelease(doc);
 }
 
-static zend_string *php_tidy_file_to_mem(char *filename, bool use_include_path)
+static zend_string *php_tidy_file_to_mem(const char *filename, bool use_include_path)
 {
 	php_stream *stream;
 	zend_string *data = NULL;
@@ -434,7 +447,7 @@ static zend_object *tidy_object_new(zend_class_entry *class_type, zend_object_ha
 				efree(intern->ptdoc->errbuf);
 				tidyRelease(intern->ptdoc->doc);
 				efree(intern->ptdoc);
-				/* TODO: convert to exception */
+				efree(intern);
 				php_error_docref(NULL, E_ERROR, "Could not set Tidy error buffer");
 			}
 
@@ -490,7 +503,7 @@ static zend_result tidy_doc_cast_handler(zend_object *in, zval *out, int type)
 			tidyBufInit(&output);
 			tidySaveBuffer (obj->ptdoc->doc, &output);
 			if (output.size) {
-				ZVAL_STRINGL(out, (char *) output.bp, output.size-1);
+				ZVAL_STRINGL(out, (const char *) output.bp, output.size-1);
 			} else {
 				ZVAL_EMPTY_STRING(out);
 			}
@@ -528,7 +541,7 @@ static zend_result tidy_node_cast_handler(zend_object *in, zval *out, int type)
 			tidyBufInit(&buf);
 			if (obj->ptdoc) {
 				tidyNodeGetText(obj->ptdoc->doc, obj->node, &buf);
-				ZVAL_STRINGL(out, (char *) buf.bp, buf.size-1);
+				ZVAL_STRINGL(out, (const char *) buf.bp, buf.size-1);
 			} else {
 				ZVAL_EMPTY_STRING(out);
 			}
@@ -580,8 +593,7 @@ static void tidy_add_node_default_properties(PHPTidyObj *obj)
 	TidyAttr	tempattr;
 	TidyNode	tempnode;
 	zval attribute, children, temp;
-	PHPTidyObj *newobj;
-	char *name;
+	const char *name;
 
 	tidyBufInit(&buf);
 	tidyNodeGetText(obj->ptdoc->doc, obj->node, &buf);
@@ -591,13 +603,13 @@ static void tidy_add_node_default_properties(PHPTidyObj *obj)
 		&obj->std,
 		"value",
 		sizeof("value") - 1,
-		buf.size ? (char *) buf.bp : "",
+		buf.size ? (const char *) buf.bp : "",
 		buf.size ? buf.size - 1 : 0
 	);
 
 	tidyBufFree(&buf);
 
-	name = (char *) tidyNodeGetName(obj->node);
+	name = (const char *) tidyNodeGetName(obj->node);
 
 	zend_update_property_string(
 		tidy_ce_node,
@@ -665,12 +677,12 @@ static void tidy_add_node_default_properties(PHPTidyObj *obj)
 	tempattr = tidyAttrFirst(obj->node);
 
 	if (tempattr) {
-		char *name, *val;
+		const char *name, *val;
 		array_init(&attribute);
 
 		do {
-			name = (char *)tidyAttrName(tempattr);
-			val = (char *)tidyAttrValue(tempattr);
+			name = (const char *)tidyAttrName(tempattr);
+			val = (const char *)tidyAttrValue(tempattr);
 			if (name) {
 				if (val) {
 					add_assoc_string(&attribute, name, val);
@@ -698,16 +710,8 @@ static void tidy_add_node_default_properties(PHPTidyObj *obj)
 	if (tempnode) {
 		array_init(&children);
 		do {
-			tidy_instantiate(tidy_ce_node, &temp);
-			newobj = Z_TIDY_P(&temp);
-			newobj->node = tempnode;
-			newobj->type = is_node;
-			newobj->ptdoc = obj->ptdoc;
-			newobj->ptdoc->ref_count++;
-
-			tidy_add_node_default_properties(newobj);
+			tidy_create_node_object(&temp, obj->ptdoc, tempnode);
 			add_next_index_zval(&children, &temp);
-
 		} while((tempnode = tidyGetNext(tempnode)));
 
 	} else {
@@ -755,7 +759,6 @@ static void *php_tidy_get_opt_val(PHPTidyDoc *ptdoc, TidyOption opt, TidyOptionT
 
 static void php_tidy_create_node(INTERNAL_FUNCTION_PARAMETERS, tidy_base_nodetypes node_type)
 {
-	PHPTidyObj *newobj;
 	TidyNode node;
 	TIDY_FETCH_OBJECT;
 
@@ -783,17 +786,10 @@ static void php_tidy_create_node(INTERNAL_FUNCTION_PARAMETERS, tidy_base_nodetyp
 		RETURN_NULL();
 	}
 
-	tidy_instantiate(tidy_ce_node, return_value);
-	newobj = Z_TIDY_P(return_value);
-	newobj->type  = is_node;
-	newobj->ptdoc = obj->ptdoc;
-	newobj->node  = node;
-	newobj->ptdoc->ref_count++;
-
-	tidy_add_node_default_properties(newobj);
+	tidy_create_node_object(return_value, obj->ptdoc, node);
 }
 
-static int _php_tidy_apply_config_array(TidyDoc doc, HashTable *ht_options)
+static int _php_tidy_apply_config_array(TidyDoc doc, const HashTable *ht_options)
 {
 	zval *opt_val;
 	zend_string *opt_name;
@@ -809,7 +805,7 @@ static int _php_tidy_apply_config_array(TidyDoc doc, HashTable *ht_options)
 	return SUCCESS;
 }
 
-static int php_tidy_parse_string(PHPTidyObj *obj, char *string, uint32_t len, char *enc)
+static int php_tidy_parse_string(PHPTidyObj *obj, const char *string, uint32_t len, const char *enc)
 {
 	TidyBuffer buf;
 
@@ -894,12 +890,12 @@ static PHP_MINFO_FUNCTION(tidy)
 	php_info_print_table_start();
 	php_info_print_table_row(2, "Tidy support", "enabled");
 #ifdef HAVE_TIDYBUFFIO_H
-	php_info_print_table_row(2, "libTidy Version", (char *)tidyLibraryVersion());
+	php_info_print_table_row(2, "libTidy Version", (const char *)tidyLibraryVersion());
 #elif defined(HAVE_TIDYP_H)
-	php_info_print_table_row(2, "libtidyp Version", (char *)tidyVersion());
+	php_info_print_table_row(2, "libtidyp Version", (const char *)tidyVersion());
 #endif
 #ifdef HAVE_TIDYRELEASEDATE
-	php_info_print_table_row(2, "libTidy Release", (char *)tidyReleaseDate());
+	php_info_print_table_row(2, "libTidy Release", (const char *)tidyReleaseDate());
 #endif
 	php_info_print_table_end();
 
@@ -962,9 +958,9 @@ static php_output_handler *php_tidy_output_handler_init(const char *handler_name
 	return php_output_handler_create_internal(handler_name, handler_name_len, php_tidy_output_handler, chunk_size, flags);
 }
 
-static int php_tidy_output_handler(void **nothing, php_output_context *output_context)
+static zend_result php_tidy_output_handler(void **nothing, php_output_context *output_context)
 {
-	int status = FAILURE;
+	zend_result status = FAILURE;
 	TidyDoc doc;
 	TidyBuffer inbuf, outbuf, errbuf;
 
@@ -1029,7 +1025,7 @@ PHP_FUNCTION(tidy_parse_string)
 	obj = Z_TIDY_P(return_value);
 
 	if (php_tidy_apply_config(obj->ptdoc->doc, options_str, options_ht) != SUCCESS
-	 || php_tidy_parse_string(obj, ZSTR_VAL(input), (uint32_t)ZSTR_LEN(input), enc) == FAILURE) {
+	 || php_tidy_parse_string(obj, ZSTR_VAL(input), (uint32_t)ZSTR_LEN(input), enc) != SUCCESS) {
 		zval_ptr_dtor(return_value);
 		RETURN_FALSE;
 	}
@@ -1042,7 +1038,7 @@ PHP_FUNCTION(tidy_get_error_buffer)
 	TIDY_FETCH_OBJECT;
 
 	if (obj->ptdoc->errbuf && obj->ptdoc->errbuf->bp) {
-		RETURN_STRINGL((char*)obj->ptdoc->errbuf->bp, obj->ptdoc->errbuf->size-1);
+		RETURN_STRINGL((const char*)obj->ptdoc->errbuf->bp, obj->ptdoc->errbuf->size-1);
 	} else {
 		RETURN_FALSE;
 	}
@@ -1058,7 +1054,7 @@ PHP_FUNCTION(tidy_get_output)
 	tidyBufInit(&output);
 	tidySaveBuffer(obj->ptdoc->doc, &output);
 	FIX_BUFFER(&output);
-	RETVAL_STRINGL((char *) output.bp, output.size ? output.size-1 : 0);
+	RETVAL_STRINGL((const char *) output.bp, output.size ? output.size-1 : 0);
 	tidyBufFree(&output);
 }
 /* }}} */
@@ -1089,7 +1085,7 @@ PHP_FUNCTION(tidy_parse_file)
 
 	if (ZEND_SIZE_T_UINT_OVFL(ZSTR_LEN(contents))) {
 		zend_string_release_ex(contents, 0);
-		zend_value_error("Input string is too long");
+		zend_value_error("File content is too long");
 		RETURN_THROWS();
 	}
 
@@ -1097,7 +1093,7 @@ PHP_FUNCTION(tidy_parse_file)
 	obj = Z_TIDY_P(return_value);
 
 	if (php_tidy_apply_config(obj->ptdoc->doc, options_str, options_ht) != SUCCESS
-	 || php_tidy_parse_string(obj, ZSTR_VAL(contents), (uint32_t)ZSTR_LEN(contents), enc) == FAILURE) {
+	 || php_tidy_parse_string(obj, ZSTR_VAL(contents), (uint32_t)ZSTR_LEN(contents), enc) != SUCCESS) {
 		zval_ptr_dtor(return_value);
 		RETVAL_FALSE;
 	}
@@ -1151,14 +1147,14 @@ PHP_FUNCTION(tidy_diagnose)
 /* {{{ Get release date (version) for Tidy library */
 PHP_FUNCTION(tidy_get_release)
 {
-	if (zend_parse_parameters_none() == FAILURE) {
+	if (zend_parse_parameters_none() != SUCCESS) {
 		RETURN_THROWS();
 	}
 
 #ifdef HAVE_TIDYRELEASEDATE
-	RETURN_STRING((char *)tidyReleaseDate());
+	RETURN_STRING((const char *)tidyReleaseDate());
 #else
-	RETURN_STRING((char *)"unknown");
+	RETURN_STRING((const char *)"unknown");
 #endif
 }
 /* }}} */
@@ -1169,12 +1165,13 @@ PHP_FUNCTION(tidy_get_release)
 PHP_FUNCTION(tidy_get_opt_doc)
 {
 	PHPTidyObj *obj;
-	char *optval, *optname;
+	const char *optval;
+	char *optname;
 	size_t optname_len;
 	TidyOption opt;
 	zval *object;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &object, tidy_ce_doc, &optname, &optname_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &object, tidy_ce_doc, &optname, &optname_len) != SUCCESS) {
 		RETURN_THROWS();
 	}
 
@@ -1183,11 +1180,11 @@ PHP_FUNCTION(tidy_get_opt_doc)
 	opt = tidyGetOptionByName(obj->ptdoc->doc, optname);
 
 	if (!opt) {
-		zend_argument_value_error(getThis() ? 1 : 2, "is an invalid configuration option, \"%s\" given", optname);
+		zend_argument_value_error(hasThis() ? 1 : 2, "is an invalid configuration option, \"%s\" given", optname);
 		RETURN_THROWS();
 	}
 
-	if ( (optval = (char *) tidyOptGetDoc(obj->ptdoc->doc, opt)) ) {
+	if ( (optval = (const char *) tidyOptGetDoc(obj->ptdoc->doc, opt)) ) {
 		RETURN_STRING(optval);
 	}
 
@@ -1201,7 +1198,7 @@ PHP_FUNCTION(tidy_get_opt_doc)
 PHP_FUNCTION(tidy_get_config)
 {
 	TidyIterator itOpt;
-	char *opt_name;
+	const char *opt_name;
 	void *opt_value;
 	TidyOptionType optt;
 
@@ -1214,7 +1211,7 @@ PHP_FUNCTION(tidy_get_config)
 	while (itOpt) {
 		TidyOption opt = tidyGetNextOption(obj->ptdoc->doc, &itOpt);
 
-		opt_name = (char *)tidyOptGetName(opt);
+		opt_name = (const char *)tidyOptGetName(opt);
 		opt_value = php_tidy_get_opt_val(obj->ptdoc, opt, &optt);
 		switch (optt) {
 			case TidyString:
@@ -1230,8 +1227,6 @@ PHP_FUNCTION(tidy_get_config)
 				break;
 		}
 	}
-
-	return;
 }
 /* }}} */
 
@@ -1318,7 +1313,7 @@ PHP_FUNCTION(tidy_getopt)
 	TidyOptionType optt;
 	zval *object;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &object, tidy_ce_doc, &optname, &optname_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Os", &object, tidy_ce_doc, &optname, &optname_len) != SUCCESS) {
 		RETURN_THROWS();
 	}
 
@@ -1327,7 +1322,7 @@ PHP_FUNCTION(tidy_getopt)
 	opt = tidyGetOptionByName(obj->ptdoc->doc, optname);
 
 	if (!opt) {
-		zend_argument_value_error(getThis() ? 1 : 2, "is an invalid configuration option, \"%s\" given", optname);
+		zend_argument_value_error(hasThis() ? 1 : 2, "is an invalid configuration option, \"%s\" given", optname);
 		RETURN_THROWS();
 	}
 
@@ -1380,21 +1375,24 @@ PHP_METHOD(tidy, __construct)
 
 	if (inputfile) {
 		if (!(contents = php_tidy_file_to_mem(ZSTR_VAL(inputfile), use_include_path))) {
-			php_error_docref(NULL, E_WARNING, "Cannot load \"%s\" into memory%s", ZSTR_VAL(inputfile), (use_include_path) ? " (using include path)" : "");
-			return;
+			zend_throw_error(zend_ce_exception, "Cannot load \"%s\" into memory%s", ZSTR_VAL(inputfile), (use_include_path) ? " (using include path)" : "");
+			RETURN_THROWS();
 		}
 
 		if (ZEND_SIZE_T_UINT_OVFL(ZSTR_LEN(contents))) {
 			zend_string_release_ex(contents, 0);
-			zend_value_error("Input string is too long");
+			zend_value_error("File content is too long");
 			RETURN_THROWS();
 		}
 
+		zend_error_handling error_handling;
+		zend_replace_error_handling(EH_THROW, NULL, &error_handling);
 		if (php_tidy_apply_config(obj->ptdoc->doc, options_str, options_ht) != SUCCESS) {
-			/* TODO: this is the constructor, we should throw probably... */
+			zend_restore_error_handling(&error_handling);
 			zend_string_release_ex(contents, 0);
-			RETURN_FALSE;
+			RETURN_THROWS();
 		}
+		zend_restore_error_handling(&error_handling);
 
 		php_tidy_parse_string(obj, ZSTR_VAL(contents), (uint32_t)ZSTR_LEN(contents), enc);
 
@@ -1429,16 +1427,12 @@ PHP_METHOD(tidy, parseFile)
 
 	if (ZEND_SIZE_T_UINT_OVFL(ZSTR_LEN(contents))) {
 		zend_string_release_ex(contents, 0);
-		zend_value_error("Input string is too long");
+		zend_value_error("File content is too long");
 		RETURN_THROWS();
 	}
 
-	if (php_tidy_apply_config(obj->ptdoc->doc, options_str, options_ht) != SUCCESS
-	 || php_tidy_parse_string(obj, ZSTR_VAL(contents), (uint32_t)ZSTR_LEN(contents), enc) == FAILURE) {
-		RETVAL_FALSE;
-	} else {
-		RETVAL_TRUE;
-	}
+	RETVAL_BOOL(php_tidy_apply_config(obj->ptdoc->doc, options_str, options_ht) == SUCCESS
+				&& php_tidy_parse_string(obj, ZSTR_VAL(contents), (uint32_t)ZSTR_LEN(contents), enc) == SUCCESS);
 
 	zend_string_release_ex(contents, 0);
 }
@@ -1466,12 +1460,8 @@ PHP_METHOD(tidy, parseString)
 	TIDY_SET_CONTEXT;
 	obj = Z_TIDY_P(object);
 
-	if (php_tidy_apply_config(obj->ptdoc->doc, options_str, options_ht) == SUCCESS
-	 && php_tidy_parse_string(obj, ZSTR_VAL(input), (uint32_t)ZSTR_LEN(input), enc) == SUCCESS) {
-		RETURN_TRUE;
-	}
-
-	RETURN_FALSE;
+	RETURN_BOOL(php_tidy_apply_config(obj->ptdoc->doc, options_str, options_ht) == SUCCESS
+				&& php_tidy_parse_string(obj, ZSTR_VAL(input), (uint32_t)ZSTR_LEN(input), enc) == SUCCESS);
 }
 
 
@@ -1613,25 +1603,34 @@ PHP_METHOD(tidyNode, isPhp)
 /* {{{ Returns the parent node if available or NULL */
 PHP_METHOD(tidyNode, getParent)
 {
-	TidyNode	parent_node;
-	PHPTidyObj *newobj;
 	TIDY_FETCH_ONLY_OBJECT;
 
-	parent_node = tidyGetParent(obj->node);
-	if(parent_node) {
-		tidy_instantiate(tidy_ce_node, return_value);
-		newobj = Z_TIDY_P(return_value);
-		newobj->node = parent_node;
-		newobj->type = is_node;
-		newobj->ptdoc = obj->ptdoc;
-		newobj->ptdoc->ref_count++;
-		tidy_add_node_default_properties(newobj);
-	} else {
-		ZVAL_NULL(return_value);
+	TidyNode parent_node = tidyGetParent(obj->node);
+	if (parent_node) {
+		tidy_create_node_object(return_value, obj->ptdoc, parent_node);
 	}
 }
 /* }}} */
 
+PHP_METHOD(tidyNode, getPreviousSibling)
+{
+	TIDY_FETCH_ONLY_OBJECT;
+
+	TidyNode previous_node = tidyGetPrev(obj->node);
+	if (previous_node) {
+		tidy_create_node_object(return_value, obj->ptdoc, previous_node);
+	}
+}
+
+PHP_METHOD(tidyNode, getNextSibling)
+{
+	TIDY_FETCH_ONLY_OBJECT;
+
+	TidyNode next_node = tidyGetNext(obj->node);
+	if (next_node) {
+		tidy_create_node_object(return_value, obj->ptdoc, next_node);
+	}
+}
 
 /* {{{ __constructor for tidyNode. */
 PHP_METHOD(tidyNode, __construct)
