@@ -133,6 +133,11 @@ const XML_OPTION_SKIP_TAGSTART = UNKNOWN;
  * @cvalue PHP_XML_OPTION_SKIP_WHITE
  */
 const XML_OPTION_SKIP_WHITE = UNKNOWN;
+/**
+ * @var int
+ * @cvalue PHP_XML_OPTION_PARSE_HUGE
+ */
+const XML_OPTION_PARSE_HUGE = UNKNOWN;
 
 /**
  * @var string
@@ -144,37 +149,26 @@ function xml_parser_create(?string $encoding = null): XMLParser {}
 
 function xml_parser_create_ns(?string $encoding = null, string $separator = ":"): XMLParser {}
 
+#[\Deprecated(since: '8.4', message: 'provide a proper method callable to xml_set_*_handler() functions')]
 function xml_set_object(XMLParser $parser, object $object): true {}
 
-/**
- * @param callable $start_handler
- * @param callable $end_handler
- */
-function xml_set_element_handler(XMLParser $parser, $start_handler, $end_handler): true {}
+function xml_set_element_handler(XMLParser $parser, callable|string|null $start_handler, callable|string|null $end_handler): true {}
 
-/** @param callable $handler */
-function xml_set_character_data_handler(XMLParser $parser, $handler): true {}
+function xml_set_character_data_handler(XMLParser $parser, callable|string|null $handler): true {}
 
-/** @param callable $handler */
-function xml_set_processing_instruction_handler(XMLParser $parser, $handler): true {}
+function xml_set_processing_instruction_handler(XMLParser $parser, callable|string|null $handler): true {}
 
-/** @param callable $handler */
-function xml_set_default_handler(XMLParser $parser, $handler): true {}
+function xml_set_default_handler(XMLParser $parser, callable|string|null $handler): true {}
 
-/** @param callable $handler */
-function xml_set_unparsed_entity_decl_handler(XMLParser $parser, $handler): true {}
+function xml_set_unparsed_entity_decl_handler(XMLParser $parser, callable|string|null $handler): true {}
 
-/** @param callable $handler */
-function xml_set_notation_decl_handler(XMLParser $parser, $handler): true {}
+function xml_set_notation_decl_handler(XMLParser $parser, callable|string|null $handler): true {}
 
-/** @param callable $handler */
-function xml_set_external_entity_ref_handler(XMLParser $parser, $handler): true {}
+function xml_set_external_entity_ref_handler(XMLParser $parser, callable|string|null $handler): true {}
 
-/** @param callable $handler */
-function xml_set_start_namespace_decl_handler(XMLParser $parser, $handler): true {}
+function xml_set_start_namespace_decl_handler(XMLParser $parser, callable|string|null $handler): true {}
 
-/** @param callable $handler */
-function xml_set_end_namespace_decl_handler(XMLParser $parser, $handler): true {}
+function xml_set_end_namespace_decl_handler(XMLParser $parser, callable|string|null $handler): true {}
 
 function xml_parse(XMLParser $parser, string $data, bool $is_final = false): int {}
 
