@@ -16,16 +16,14 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 
 #include "php.h"
-#include "php_ini.h"
 #include "ext/standard/info.h"
 #include "php_xmlwriter.h"
 #include "php_xmlwriter_arginfo.h"
-#include "ext/standard/php_string.h"
 
 static zend_class_entry *xmlwriter_class_entry_ce;
 
@@ -46,12 +44,8 @@ typedef int (*xmlwriter_read_int_t)(xmlTextWriterPtr writer);
 
 static zend_object_handlers xmlwriter_object_handlers;
 
-/* {{{{ xmlwriter_object_dtor */
-static void xmlwriter_object_dtor(zend_object *object)
+static zend_always_inline void xmlwriter_destroy_libxml_objects(ze_xmlwriter_object *intern)
 {
-	ze_xmlwriter_object *intern = php_xmlwriter_fetch_object(object);
-
-	/* freeing the resource here may leak, but otherwise we may use it after it has been freed */
 	if (intern->ptr) {
 		xmlFreeTextWriter(intern->ptr);
 		intern->ptr = NULL;
@@ -60,6 +54,15 @@ static void xmlwriter_object_dtor(zend_object *object)
 		xmlBufferFree(intern->output);
 		intern->output = NULL;
 	}
+}
+
+/* {{{{ xmlwriter_object_dtor */
+static void xmlwriter_object_dtor(zend_object *object)
+{
+	ze_xmlwriter_object *intern = php_xmlwriter_fetch_object(object);
+
+	/* freeing the resource here may leak, but otherwise we may use it after it has been freed */
+	xmlwriter_destroy_libxml_objects(intern);
 	zend_objects_destroy_object(object);
 }
 /* }}} */
@@ -154,7 +157,7 @@ static char *_xmlwriter_get_valid_file_path(char *source, char *resolved_path, i
 		}
 
 		memcpy(file_dirname, source, strlen(source));
-		dir_len = php_dirname(file_dirname, strlen(source));
+		dir_len = zend_dirname(file_dirname, strlen(source));
 
 		if (dir_len > 0) {
 			zend_stat_t buf = {0};
@@ -175,9 +178,27 @@ static char *_xmlwriter_get_valid_file_path(char *source, char *resolved_path, i
 }
 /* }}} */
 
+static void xml_writer_create_static(INTERNAL_FUNCTION_PARAMETERS, xmlTextWriterPtr writer, xmlBufferPtr output)
+{
+	if (object_init_with_constructor(return_value, Z_CE_P(ZEND_THIS), 0, NULL, NULL) == SUCCESS) {
+		ze_xmlwriter_object *intern = Z_XMLWRITER_P(return_value);
+		intern->ptr = writer;
+		intern->output = output;
+	} else {
+		xmlBufferFree(output);
+		xmlFreeTextWriter(writer);
+	}
+}
+
+static const zend_module_dep xmlwriter_deps[] = {
+	ZEND_MOD_REQUIRED("libxml")
+	ZEND_MOD_END
+};
+
 /* {{{ xmlwriter_module_entry */
 zend_module_entry xmlwriter_module_entry = {
-	STANDARD_MODULE_HEADER,
+	STANDARD_MODULE_HEADER_EX, NULL,
+	xmlwriter_deps,
 	"xmlwriter",
 	ext_functions,
 	PHP_MINIT(xmlwriter),
@@ -578,7 +599,7 @@ PHP_FUNCTION(xmlwriter_start_document)
 	int retval;
 	zval *self;
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O|s!s!s!", &self, xmlwriter_class_entry_ce, &version, &version_len, &enc, &enc_len, &alone, &alone_len) == FAILURE) {
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O|s!p!s!", &self, xmlwriter_class_entry_ce, &version, &version_len, &enc, &enc_len, &alone, &alone_len) == FAILURE) {
 		RETURN_THROWS();
 	}
 	XMLWRITER_FROM_OBJECT(ptr, self);
@@ -796,7 +817,7 @@ PHP_FUNCTION(xmlwriter_open_uri)
 	}
 
 	if (source_len == 0) {
-		zend_argument_value_error(1, "cannot be empty");
+		zend_argument_must_not_be_empty_error(1);
 		RETURN_THROWS();
 	}
 
@@ -813,12 +834,7 @@ PHP_FUNCTION(xmlwriter_open_uri)
 	}
 
 	if (self) {
-		if (ze_obj->ptr) {
-			xmlFreeTextWriter(ze_obj->ptr);
-		}
-		if (ze_obj->output) {
-			xmlBufferFree(ze_obj->output);
-		}
+		xmlwriter_destroy_libxml_objects(ze_obj);
 		ze_obj->ptr = ptr;
 		ze_obj->output = NULL;
 		RETURN_TRUE;
@@ -831,6 +847,36 @@ PHP_FUNCTION(xmlwriter_open_uri)
 }
 /* }}} */
 
+PHP_METHOD(XMLWriter, toUri)
+{
+	char *source;
+	size_t source_len;
+	char resolved_path[MAXPATHLEN + 1];
+
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_PATH(source, source_len)
+	ZEND_PARSE_PARAMETERS_END();
+
+	if (source_len == 0) {
+		zend_argument_must_not_be_empty_error(1);
+		RETURN_THROWS();
+	}
+
+	const char *valid_file = _xmlwriter_get_valid_file_path(source, resolved_path, MAXPATHLEN);
+	if (!valid_file) {
+		zend_argument_value_error(1, "must resolve to a valid file path");
+		RETURN_THROWS();
+	}
+
+	xmlTextWriterPtr writer = xmlNewTextWriterFilename(valid_file, 0);
+	if (!writer) {
+		zend_throw_error(NULL, "Could not construct libxml writer");
+		RETURN_THROWS();
+	}
+
+	xml_writer_create_static(INTERNAL_FUNCTION_PARAM_PASSTHRU, writer, NULL);
+}
+
 /* {{{ Create new xmlwriter using memory for string output */
 PHP_FUNCTION(xmlwriter_open_memory)
 {
@@ -862,12 +908,7 @@ PHP_FUNCTION(xmlwriter_open_memory)
 	}
 
 	if (self) {
-		if (ze_obj->ptr) {
-			xmlFreeTextWriter(ze_obj->ptr);
-		}
-		if (ze_obj->output) {
-			xmlBufferFree(ze_obj->output);
-		}
+		xmlwriter_destroy_libxml_objects(ze_obj);
 		ze_obj->ptr = ptr;
 		ze_obj->output = buffer;
 		RETURN_TRUE;
@@ -881,6 +922,73 @@ PHP_FUNCTION(xmlwriter_open_memory)
 }
 /* }}} */
 
+PHP_METHOD(XMLWriter, toMemory)
+{
+	ZEND_PARSE_PARAMETERS_NONE();
+
+	xmlBufferPtr buffer = xmlBufferCreate();
+	xmlTextWriterPtr writer = xmlNewTextWriterMemory(buffer, 0);
+
+	/* No need for an explicit buffer check as this will fail on a NULL buffer. */
+	if (!writer) {
+		xmlBufferFree(buffer);
+		zend_throw_error(NULL, "Could not construct libxml writer");
+		RETURN_THROWS();
+	}
+
+	xml_writer_create_static(INTERNAL_FUNCTION_PARAM_PASSTHRU, writer, buffer);
+}
+
+static int xml_writer_stream_write(void *context, const char *buffer, int len)
+{
+	zend_resource *resource = context;
+	if (EXPECTED(resource->ptr)) {
+		php_stream *stream = resource->ptr;
+		return php_stream_write(stream, buffer, len);
+	}
+	return -1;
+}
+
+static int xml_writer_stream_close(void *context)
+{
+	zend_resource *resource = context;
+	/* Don't close it as others may still use it! We don't own the resource!
+	 * Just delete our reference (and clean up if we're the last one). */
+	zend_list_delete(resource);
+	return 0;
+}
+
+PHP_METHOD(XMLWriter, toStream)
+{
+	zval *stream_zv;
+	php_stream *stream;
+
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_RESOURCE(stream_zv)
+	ZEND_PARSE_PARAMETERS_END();
+
+	php_stream_from_res(stream, Z_RES_P(stream_zv));
+
+	xmlOutputBufferPtr output_buffer = xmlOutputBufferCreateIO(xml_writer_stream_write, xml_writer_stream_close, stream->res, NULL);
+	if (UNEXPECTED(output_buffer == NULL)) {
+		zend_throw_error(NULL, "Could not construct libxml output buffer");
+		RETURN_THROWS();
+	}
+
+	/* When the buffer is closed (even in error paths) the reference is destroyed. */
+	Z_ADDREF_P(stream_zv);
+
+	xmlTextWriterPtr writer = xmlNewTextWriter(output_buffer);
+	if (UNEXPECTED(writer == NULL)) {
+		xmlOutputBufferClose(output_buffer);
+		zend_throw_error(NULL, "Could not construct libxml writer");
+		RETURN_THROWS();
+	}
+
+	/* output_buffer is owned by writer, and so writer will clean that up for us. */
+	xml_writer_create_static(INTERNAL_FUNCTION_PARAM_PASSTHRU, writer, NULL);
+}
+
 /* {{{ php_xmlwriter_flush */
 static void php_xmlwriter_flush(INTERNAL_FUNCTION_PARAMETERS, int force_string) {
 	xmlTextWriterPtr ptr;
@@ -900,7 +1008,8 @@ static void php_xmlwriter_flush(INTERNAL_FUNCTION_PARAMETERS, int force_string)
 	}
 	output_bytes = xmlTextWriterFlush(ptr);
 	if (buffer) {
-		RETVAL_STRING((char *) buffer->content);
+		const xmlChar *content = xmlBufferContent(buffer);
+		RETVAL_STRING((const char *) content);
 		if (empty) {
 			xmlBufferEmpty(buffer);
 		}
