@@ -29,7 +29,6 @@
 #include <winbase.h>
 #include "sendmail.h"
 #include "php_ini.h"
-#include "inet.h"
 
 #include "php_win32_globals.h"
 
@@ -87,7 +86,7 @@ static char *ErrorMessages[] =
 	{"Bad Message File"},
 	{"\"sendmail_from\" not set in php.ini or custom \"From:\" header missing"},
 	{"Mailserver rejected our \"sendmail_from\" setting"}, /* 20 */
-	{"Error while trimming mail header with PCRE, please file a bug report at http://bugs.php.net/"} /* 21 */
+	{"Error while trimming mail header with PCRE, please file a bug report at https://github.com/php/php-src/issues"} /* 21 */
 };
 
 /* This pattern converts all single occurrences of \n (Unix)
@@ -163,7 +162,7 @@ static zend_string *php_win32_mail_trim_header(const char *header)
 	return result2;
 }
 
-/*********************************************************************
+//*********************************************************************
 // Name:  TSendMail
 // Input:   1) host:    Name of the mail host where the SMTP server resides
 //                      max accepted length of name = 256
@@ -175,7 +174,7 @@ static zend_string *php_win32_mail_trim_header(const char *header)
 //                      SUCCESS otherwise.
 //
 //  See SendText() for additional args!
-//********************************************************************/
+//*********************************************************************
 PHPAPI int TSendMail(const char *host, int *error, char **error_message,
 			  const char *headers, const char *Subject, const char *mailTo, const char *data,
 			  char *mailCc, char *mailBcc, char *mailRPath)
@@ -292,14 +291,14 @@ PHPAPI int TSendMail(const char *host, int *error, char **error_message,
 	}
 }
 
-//********************************************************************
+//*********************************************************************
 // Name:  TSendMail::~TSendMail
 // Input:
 // Output:
 // Description: DESTRUCTOR
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************/
+//*********************************************************************
 PHPAPI void TSMClose(void)
 {
 	Post("QUIT\r\n");
@@ -314,14 +313,14 @@ PHPAPI void TSMClose(void)
 }
 
 
-/*********************************************************************
+//*********************************************************************
 // Name:  char *GetSMErrorText
 // Input:   Error index returned by the member functions
 // Output:  pointer to a string containing the error description
 // Description:
 // Author/Date:  jcar 20/9/96
 // History:
-//*******************************************************************/
+//*********************************************************************
 PHPAPI char *GetSMErrorText(int index)
 {
 	if (MIN_ERROR_INDEX <= index && index < MAX_ERROR_INDEX) {
@@ -365,7 +364,7 @@ static char *find_address(char *list, char **state)
 	return list;
 }
 
-/*********************************************************************
+//*********************************************************************
 // Name:  SendText
 // Input:       1) RPath:   return path of the message
 //                                  Is used to fill the "Return-Path" and the
@@ -382,7 +381,7 @@ static char *find_address(char *list, char **state)
 // Description:
 // Author/Date:  jcar 20/9/96
 // History:
-//*******************************************************************/
+//*********************************************************************
 static int SendText(char *RPath, const char *Subject, const char *mailTo, char *mailCc, char *mailBcc, const char *data,
 			 const char *headers, char *headers_lc, char **error_message)
 {
@@ -692,12 +691,14 @@ static int SendText(char *RPath, const char *Subject, const char *mailTo, char *
 
 static int addToHeader(char **header_buffer, const char *specifier, const char *string)
 {
-	*header_buffer = erealloc(*header_buffer, strlen(*header_buffer) + strlen(specifier) + strlen(string) + 1);
-	sprintf(*header_buffer + strlen(*header_buffer), specifier, string);
+	size_t header_buffer_size = strlen(*header_buffer);
+	size_t total_size = header_buffer_size + strlen(specifier) + strlen(string) + 1;
+	*header_buffer = erealloc(*header_buffer, total_size);
+	snprintf(*header_buffer + header_buffer_size, total_size - header_buffer_size, specifier, string);
 	return 1;
 }
 
-/*********************************************************************
+//*********************************************************************
 // Name:  PostHeader
 // Input:       1) return path
 //              2) Subject
@@ -707,7 +708,7 @@ static int addToHeader(char **header_buffer, const char *specifier, const char *
 // Description:
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************/
+//*********************************************************************
 static int PostHeader(char *RPath, const char *Subject, const char *mailTo, char *xheaders)
 {
 	/* Print message header according to RFC 822 */
@@ -784,14 +785,14 @@ PostHeader_outofmem:
 
 
 
-/*********************************************************************
+//*********************************************************************
 // Name:  MailConnect
 // Input:   None
 // Output:  None
 // Description: Connect to the mail host and receive the welcome message.
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************/
+//*********************************************************************
 static int MailConnect()
 {
 
@@ -880,14 +881,14 @@ return 0;
 }
 
 
-/*********************************************************************
+//*********************************************************************
 // Name:  Post
 // Input:
 // Output:
 // Description:
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************/
+//*********************************************************************
 static int Post(LPCSTR msg)
 {
 	int len = (int)strlen(msg);
@@ -911,7 +912,7 @@ static int Post(LPCSTR msg)
 
 
 
-/*********************************************************************
+//*********************************************************************
 // Name:  Ack
 // Input:
 // Output:
@@ -920,7 +921,7 @@ static int Post(LPCSTR msg)
 // last command was successful.
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************/
+//*********************************************************************
 static int Ack(char **server_response)
 {
 	ZEND_TLS char buf[MAIL_BUFFER_SIZE];
@@ -973,7 +974,7 @@ again:
 }
 
 
-/*********************************************************************
+//*********************************************************************
 // Name:  unsigned long GetAddr (LPSTR szHost)
 // Input:
 // Output:
@@ -984,7 +985,7 @@ again:
 // WARNING: gethostbyname() is a blocking function
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************/
+//*********************************************************************
 static unsigned long GetAddr(LPSTR szHost)
 {
 	LPHOSTENT lpstHost;
@@ -1051,7 +1052,7 @@ static char *get_angle_addr(char *address)
 	return estrndup(p1, p2 - p1);
 }
 
-/*********************************************************************
+//*********************************************************************
 // Name:  int FormatEmailAddress
 // Input:
 // Output:
@@ -1063,7 +1064,7 @@ static char *get_angle_addr(char *address)
 //
 // Author/Date:  garretts 08/18/2009
 // History:
-//********************************************************************/
+//*********************************************************************
 static int FormatEmailAddress(char* Buf, char* EmailAddress, char* FormatString) {
 	char *tmpAddress;
 	int result;
